/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= */
/*
 *
 *  Created 18-DEC-1998       Richard B. Johnson   Analogic Corporation
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <utime.h>
#include <wait.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <mailer.h>
#define _DEBUG

#ifdef DEBUG
#define DEB(f) f
#else
#define DEB(f)
#endif


static void quit(int sig)
{
    fprintf(stdout, "Aborted via ^%c\n", ('A' + sig));
    (void)fflush(stdout);
    exit(EXIT_SUCCESS);
}
static const char usage[]="Usage:\n%s <hostname>\n";
static const char message[]="Subject: SPAM\r\n"\
     "Your server is sending SPAM to my machine. You\r\n"\
     "are  stealing  my  resources with unauthorized\r\n"\
     "access to  this  machine.  I am  maintaining a\r\n"\
     "log of these  unauthorized operations.  Please\r\n"\
     "stop this criminal activity immediately.\r\n.\r\n";


static const char *names[]={ "-administrator@",
                             "root@localhost",
                             "root@loghost",
                             "postmaster@localhost",
                             "postmaster@loghost",
                             "postmaster@mailhost",
                             "-postmaster@",
                             "-abuse@",
                             "-root@" };

#define ArraySize(a) (sizeof(a) / sizeof(a[0]))
 
int main(int args, char *argv[])
{
    char hostnam[MAXHOSTNAMELEN];
    char domain[MAXHOSTNAMELEN];
    char *buffer;
    struct hostent *host;                    /* Server address pointer  */
    struct hostent *client;
    struct sockaddr_in cli_addr;             /* Client address struct   */
    struct sockaddr_in ip_sock;              /* IP socket address       */
    struct sigaction sa;
    int s, i, n;
    if(args !=  2)
    {
        DEB(fprintf(stderr, usage, argv[0]));
        exit(EXIT_FAILURE);
    }

    show("Standby...");
    if((buffer = CP malloc(BUF_LEN * sizeof(char))) == NULL)
        ERRORS(errno);
    gethostname(hostnam, sizeof(hostnam));
    getdomainname(domain, sizeof(domain));
/*
 *  Set up for orderly shutdown from ^C.
 */
    if(sigaction(SIGINT, NULL, &sa) < 0)
        ERRORS(errno);
    sa.sa_flags = SA_INTERRUPT;
    sa.sa_handler = quit;
    if(sigaction(SIGINT, &sa, NULL) < 0)
        ERRORS(errno);
    if((host = gethostbyname(argv[1])) == NULL)
        ERRORS(ENONET);
    if((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        ERRORS(errno);
    i = 1; 
    if((setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i))) < 0)
        ERRORS(errno);
    memset(&ip_sock, 0x00, sizeof(ip_sock));
    ip_sock.sin_addr.s_addr = *((long *)host->h_addr);
    ip_sock.sin_family = AF_INET;
    ip_sock.sin_port = htons(PORT);
    if(connect(s, SSAP &ip_sock, sizeof(ip_sock)) < 0)
        ERRORS(errno);

    show("Connected");
    i = sizeof(cli_addr);
    show("Get address of client");
    if(getpeername(s, SSAP &cli_addr, &i) < 0)
        ERRORS(errno);
    show("Get name of client");
    if((client = gethostbyaddr(CP &cli_addr.sin_addr, 
                          sizeof(cli_addr.sin_addr), AF_INET)) == NULL)
        ERRORS(errno);
    DEB(fprintf(stdout, "Connected to %s\n", client->h_name));
    if(read(s, buffer, BUF_LEN) <=0)
    {
        close(s);
        free(buffer);
        return 0;
    }
    if(atoi(buffer) != 220)
    {
        close(s);
        free(buffer);
        return 0;
    }
    i = sprintf(buffer, "HELO %s.%s\r\n", hostnam, domain); 
    DEB(printf(buffer));
    if(write(s, buffer, i) != i)
    {
        close(s);
        free(buffer);
        return 0;
    }
    if((i = read(s, buffer, BUF_LEN)) <=0)
    {
        close(s);
        free(buffer);
        return 0;
    }
    buffer[i] = (char) 0x00;
    if(atoi(buffer) != 250)
    {
        close(s);
        free(buffer);
        return 0;
    }
    DEB(printf(buffer));
    i = sprintf(buffer, "MAIL FROM: root@%s.%s\r\n", hostnam, domain); 
    DEB(printf(buffer));
    if(write(s, buffer, i) != i)
    {
        close(s);
        free(buffer);
        return 0;
    }
    if((i = read(s, buffer, BUF_LEN)) <=0)
    {
        close(s);
        free(buffer);
        return 0;
    }
    buffer[i] = (char) 0x00;
    if(atoi(buffer) != 250)
    {
        close(s);
        free(buffer);
        return 0;
    }
    DEB(printf(buffer));
    for(n = 0; n< ArraySize(names); n++)
    {
        if(*names[n] != (char) '-')
            i = sprintf(buffer, "RCPT TO: %s\r\n", names[n]);
        else
            i = sprintf(buffer, "RCPT TO: %s%s\r\n", &names[n][1], argv[1]);
        DEB(printf(buffer));
        if(write(s, buffer, i) != i)
        {
            close(s);
            free(buffer);
            return 0;
        }
        if((i = read(s, buffer, BUF_LEN)) <=0)
        {
            close(s);
            free(buffer);
            return 0;
        }
        buffer[i] = (char) 0x00;
        DEB(printf(buffer));
        if(atoi(buffer) == 250)
            goto okay;
    } 
    close(s);
    free(buffer);
    return 0;
    okay:;
    i = sprintf(buffer, "DATA\r\n");
    if(write(s, buffer, i) != i)
    {
        close(s);
        free(buffer);
        return 0;
    }
    if((i = read(s, buffer, BUF_LEN)) <=0)
    {
        close(s);
        free(buffer);
        return 0;
    }
    i = sprintf(buffer, "%s", message);
    if(write(s, buffer, i) != i)
    {
        close(s);
        free(buffer);
        return 0;
    }
    if((i = read(s, buffer, BUF_LEN)) <=0)
    {
        close(s);
        free(buffer);
        return 0;
    }
    buffer[i] = (char) 0x00;
    DEB(printf(buffer));
    if(atoi(buffer) != 250)
        puts("Failed");
    i = sprintf(buffer, "QUIT\r\n");
    if(write(s, buffer, i) != i)
    {
        close(s);
        free(buffer);
        return 0;
    }
    if((i = read(s, buffer, BUF_LEN)) <=0)
    {
        close(s);
        free(buffer);
        return 0;
    }
    buffer[i] = (char) 0x00;
    DEB(printf(buffer));
    free(buffer);
    close(s);
    return 0;
}
/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= */
/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= */
/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= */
